@@ -38,6 +38,9 @@
 #include "terminal/color.h"
 #include "terminal/screen.h"
 #include "terminal/terminal.h"
+#ifdef CONFIG_TERMINFO
+#include "terminal/terminfo.h"
+#endif
 #include "util/color.h"
 #include "util/error.h"
 #include "util/memory.h"
@@ -782,13 +785,16 @@ register_autocreated_options(void)
 	get_opt_int("terminal.vt110.type", NULL) = TERM_VT100;
 	get_opt_int("terminal.xterm.type", NULL) = TERM_VT100;
 	get_opt_bool("terminal.xterm.underline", NULL) = 1;
+	get_opt_bool("terminal.xterm.strike", NULL) = 1;
 	get_opt_int("terminal.xterm-color.type", NULL) = TERM_VT100;
 	get_opt_int("terminal.xterm-color.colors", NULL) = COLOR_MODE_16;
 	get_opt_bool("terminal.xterm-color.underline", NULL) = 1;
+	get_opt_bool("terminal.xterm-color.strike", NULL) = 1;
 #ifdef CONFIG_88_COLORS
 	get_opt_int("terminal.xterm-88color.type", NULL) = TERM_VT100;
 	get_opt_int("terminal.xterm-88color.colors", NULL) = COLOR_MODE_88;
 	get_opt_bool("terminal.xterm-88color.underline", NULL) = 1;
+	get_opt_bool("terminal.xterm-88color.strike", NULL) = 1;
 #endif
 	get_opt_int("terminal.rxvt-unicode.type", NULL) = 1;
 #ifdef CONFIG_88_COLORS
@@ -798,6 +804,7 @@ register_autocreated_options(void)
 #endif
 	get_opt_bool("terminal.rxvt-unicode.italic", NULL) = 1;
 	get_opt_bool("terminal.rxvt-unicode.underline", NULL) = 1;
+	get_opt_bool("terminal.rxvt-unicode.strike", NULL) = 0;
 #ifdef CONFIG_256_COLORS
 	get_opt_int("terminal.xterm-256color.type", NULL) = TERM_VT100;
 	get_opt_int("terminal.xterm-256color.colors", NULL) = COLOR_MODE_256;
@@ -805,6 +812,7 @@ register_autocreated_options(void)
 	get_opt_int("terminal.fbterm.type", NULL) = TERM_FBTERM;
 	get_opt_int("terminal.fbterm.colors", NULL) = COLOR_MODE_256;
 	get_opt_bool("terminal.fbterm.underline", NULL) = 0;
+	get_opt_bool("terminal.fbterm.strike", NULL) = 0;
 #endif
 	get_opt_int("terminal.st-256color.type", NULL) = TERM_VT100;
 	get_opt_bool("terminal.st-256color.latin1_title", NULL) = 0;
@@ -819,6 +827,7 @@ register_autocreated_options(void)
 #endif
 	get_opt_bool("terminal.st-256color.italic", NULL) = 1;
 	get_opt_bool("terminal.st-256color.underline", NULL) = 1;
+	get_opt_bool("terminal.st-256color.strike", NULL) = 1;
 }
 
 extern union option_info cmdline_options_info[];
@@ -886,7 +895,8 @@ change_hook_ui_double_esc(struct session *ses, struct option *current, struct op
 static int
 change_hook_ui_mouse_disable(struct session *ses, struct option *current, struct option *changed)
 {
-	char *lock_filename = straconcat(empty_string_or_(elinks_home), "mouse.lock", (char *)NULL);
+	char *xdg_config_home = get_xdg_config_home();
+	char *lock_filename = straconcat(empty_string_or_(xdg_config_home), "mouse.lock", (char *)NULL);
 
 	if (lock_filename) {
 		if (changed->value.number) {
@@ -1420,3 +1430,44 @@ get_default_protocol(void)
 {
 	return get_opt_str("protocol.default_protocol", NULL);
 }
+
+color_mode_T
+get_color_mode(struct option *term_spec)
+{
+#ifdef CONFIG_TERMINFO
+	if (get_cmd_opt_bool("terminfo")) {
+		int max_colors = terminfo_max_colors();
+		color_mode_T color_mode = COLOR_MODE_16;
+
+		switch (max_colors) {
+		case 88:
+#ifdef CONFIG_88_COLORS
+			color_mode = COLOR_MODE_88;
+#endif
+			break;
+
+		case 256:
+#ifdef CONFIG_256_COLORS
+			color_mode = COLOR_MODE_256;
+#endif
+			break;
+
+		case 16:
+		case 8:
+			break;
+		default:
+#ifdef CONFIG_TRUE_COLOR
+			if (max_colors > 256) {
+				color_mode = COLOR_MODE_TRUE_COLOR;
+			} else
+#endif
+			{
+				color_mode = COLOR_MODE_MONO;
+			}
+			break;
+		}
+		return color_mode;
+	}
+#endif
+	return get_opt_int_tree(term_spec, "colors", NULL);
+}
