@@ -5,6 +5,7 @@
 #endif
 
 #include <ctype.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -25,6 +26,9 @@
 #include "protocol/uri.h"
 #include "terminal/color.h"
 #include "terminal/draw.h"
+#ifdef CONFIG_LIBSIXEL
+#include "terminal/sixel.h"
+#endif
 #include "util/color.h"
 #include "util/error.h"
 #include "util/memory.h"
@@ -54,6 +58,10 @@ struct plain_renderer {
 
 	/* Are we doing line compression */
 	unsigned int compress:1;
+
+#ifdef CONFIG_LIBSIXEL
+	unsigned int sixel:1;
+#endif
 };
 
 #define realloc_document_links(doc, size) \
@@ -67,13 +75,13 @@ realloc_line(struct document *document, int x, int y)
 	if (!line) return NULL;
 
 	if (x != line->length) {
-		if (!ALIGN_LINE(&line->chars, line->length, x))
+		if (!ALIGN_LINE(&line->ch.chars, line->length, x))
 			return NULL;
 
 		line->length = x;
 	}
 
-	return line->chars;
+	return line->ch.chars;
 }
 
 static inline struct link *
@@ -560,7 +568,7 @@ add_document_line(struct plain_renderer *renderer,
 
 		prev_char = line_pos > 0 ? line[line_pos - 1] : '\0';
 		next_char = (line_pos + charlen < width) ?
-		  		line[line_pos + charlen] : '\0';
+				line[line_pos + charlen] : '\0';
 
 		/* Do not expand tabs that precede back-spaces; this saves the
 		 * back-space code some trouble. */
@@ -639,10 +647,47 @@ add_document_line(struct plain_renderer *renderer,
 			if (template_->attr)
 				template_->attr |= pos->attr;
 		} else if (line_char == 27) {
-			decode_esc_color(line, &line_pos, width,
+#ifdef CONFIG_LIBSIXEL
+			if (renderer->sixel && line_pos + 1 < width && line[line_pos + 1] == 'P') { // && line_pos + 2 < width && line[line_pos + 2] == 'q') {
+				while (1) {
+					char *end = (char *)memchr(line + line_pos + 1, 27, width - line_pos - 1);
+
+					if (end == NULL) {
+						break;
+					}
+					if (end[1] == '\\') {
+						struct string pixels;
+						int how_many;
+
+						if (!init_string(&pixels)) {
+							break;
+						}
+						add_bytes_to_string(&pixels, line + line_pos, end + 2 - line - line_pos);
+						how_many = add_image_to_document(document, &pixels, lineno) + 1;
+						done_string(&pixels);
+
+						realloc_line(document, pos - startpos, lineno);
+
+						for (int i = 0; i < how_many; i++) {
+							realloc_line(document, 0, lineno + i);
+						}
+						renderer->lineno += how_many;
+						lineno += how_many;
+						line_pos = end + 2 - line;
+						startpos = pos = realloc_line(document, width, lineno);
+						goto zero;
+					} else {
+						line_pos = end - line;
+					}
+				}
+			} else
+#endif
+			{
+				decode_esc_color(line, &line_pos, width,
 					 &saved_renderer_template,
 					 doc_opts->color_mode, &was_reversed);
-			*template_ = saved_renderer_template;
+				*template_ = saved_renderer_template;
+			}
 		} else {
 			int added_chars = 0;
 
@@ -700,6 +745,10 @@ add_document_line(struct plain_renderer *renderer,
 next:
 		line_pos += charlen;
 		cells += cell;
+#ifdef CONFIG_LIBSIXEL
+zero:
+	;
+#endif
 	}
 	mem_free(line);
 
@@ -749,11 +798,11 @@ add_document_lines(struct plain_renderer *renderer)
 		int last_space = 0;
 		int tab_spaces = 0;
 		int step = 0;
- 		int cells = 0;
+		int cells = 0;
 
 		/* End of line detection: We handle \r, \r\n and \n types. */
- 		for (width = 0; (width < length) &&
- 				(cells < renderer->max_width);) {
+		for (width = 0; (width < length) &&
+				(cells < renderer->max_width);) {
 			if (source[width] == ASCII_CR)
 				step++;
 			if (source[width + step] == ASCII_LF)
@@ -859,20 +908,20 @@ fixup_tables(struct plain_renderer *renderer)
 		for (x = 0; x < line->length; x++) {
 			int dir;
 #ifdef CONFIG_UTF8
-			unicode_val_T ch = line->chars[x].data;
+			unicode_val_T ch = line->ch.chars[x].data;
 			unicode_val_T prev_char, next_char, up_char, down_char;
 #else
-			unsigned char ch = line->chars[x].data;
+			unsigned char ch = line->ch.chars[x].data;
 			unsigned char prev_char, next_char, up_char, down_char;
 #endif
 			if (ch != '+' && ch != '-' && ch != '|') {
 				continue;
 			}
 
-			prev_char = x > 0 ? line->chars[x - 1].data : ' ';
-			next_char = x < line->length - 1 ? line->chars[x + 1].data : ' ';
-			up_char = (prev_line && x < prev_line->length) ? prev_line->chars[x].data : ' ';
-			down_char = (next_line && x < next_line->length) ? next_line->chars[x].data : ' ';
+			prev_char = x > 0 ? line->ch.chars[x - 1].data : ' ';
+			next_char = x < line->length - 1 ? line->ch.chars[x + 1].data : ' ';
+			up_char = (prev_line && x < prev_line->length) ? prev_line->ch.chars[x].data : ' ';
+			down_char = (next_line && x < next_line->length) ? next_line->ch.chars[x].data : ' ';
 
 			switch (ch) {
 			case '+':
@@ -884,40 +933,40 @@ fixup_tables(struct plain_renderer *renderer)
 
 				switch (dir) {
 				case 15:
-					line->chars[x].data = BORDER_SCROSS;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SCROSS;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 					break;
 				case 13:
-					line->chars[x].data = BORDER_SLTEE;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SLTEE;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 					break;
 				case 7:
-					line->chars[x].data = BORDER_SRTEE;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SRTEE;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 					break;
 				case 6:
-					line->chars[x].data = BORDER_SULCORNER;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SULCORNER;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 					break;
 				case 12:
-					line->chars[x].data = BORDER_SURCORNER;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SURCORNER;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 					break;
 				case 3:
-					line->chars[x].data = BORDER_SDLCORNER;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SDLCORNER;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 					break;
 				case 9:
-					line->chars[x].data = BORDER_SDRCORNER;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SDRCORNER;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 					break;
 				case 11:
-					line->chars[x].data = BORDER_SUTEE;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SUTEE;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 					break;
 				case 14:
-					line->chars[x].data = BORDER_SDTEE;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SDTEE;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 					break;
 				default:
 					break;
@@ -927,8 +976,8 @@ fixup_tables(struct plain_renderer *renderer)
 				if (prev_char == BORDER_SHLINE || prev_char == BORDER_SCROSS || prev_char == '+' || prev_char == '|'
 				|| prev_char == BORDER_SULCORNER || prev_char == BORDER_SDLCORNER || prev_char == BORDER_SRTEE
 				|| prev_char == BORDER_SUTEE || prev_char == BORDER_SDTEE) {
-					line->chars[x].data = BORDER_SHLINE;
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].data = BORDER_SHLINE;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 				}
 				break;
 			case '|':
@@ -936,13 +985,13 @@ fixup_tables(struct plain_renderer *renderer)
 				|| up_char == BORDER_SURCORNER || up_char == BORDER_SCROSS || up_char == BORDER_SRTEE || up_char == BORDER_SLTEE
 				|| up_char == BORDER_SDTEE) {
 					if (next_char == '-') {
-						line->chars[x].data = BORDER_SRTEE;
+						line->ch.chars[x].data = BORDER_SRTEE;
 					} else if (prev_char == BORDER_SHLINE || prev_char == '-') {
-						line->chars[x].data = BORDER_SLTEE;
+						line->ch.chars[x].data = BORDER_SLTEE;
 					} else {
-						line->chars[x].data = BORDER_SVLINE;
+						line->ch.chars[x].data = BORDER_SVLINE;
 					}
-					line->chars[x].attr = SCREEN_ATTR_FRAME;
+					line->ch.chars[x].attr = SCREEN_ATTR_FRAME;
 				}
 				break;
 			default:
@@ -973,6 +1022,9 @@ render_plain_document(struct cache_entry *cached, struct document *document,
 	renderer.lineno = 0;
 	renderer.convert_table = convert_table;
 	renderer.compress = document->options.plain_compress_empty_lines;
+#ifdef CONFIG_LIBSIXEL
+	renderer.sixel = document->options.sixel;
+#endif
 	renderer.max_width = document->options.wrap ? document->options.document_width
 						    : INT_MAX;
 
