@@ -3,8 +3,16 @@
 
 #include <mujs.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 void mjs_push_implementation(js_State *J);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
 
 
