@@ -3,8 +3,16 @@
 
 #include <mujs.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct form_state;
 
 void mjs_push_input_object(js_State *J, struct form_state *fs);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
