@@ -3,6 +3,14 @@
 
 #include <mujs.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int mjs_localstorage_init(js_State *J);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
