@@ -3,6 +3,17 @@
 
 #include <quickjs/quickjs.h>
 
+#if !defined(JS_NAN_BOXING) && defined(__cplusplus)
+inline int operator<(JSValueConst a, JSValueConst b)
+{
+	return JS_VALUE_GET_PTR(a) < JS_VALUE_GET_PTR(b);
+}
+#endif
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #ifdef ECMASCRIPT_DEBUG
 
 #define RETURN_JS(obj) \
@@ -13,9 +24,17 @@
     } \
 	return obj
 
+#define REF_JS(obj) \
+	fprintf(stderr, "%s:%d obj=%p\n", __FILE__, __LINE__, JS_VALUE_GET_PTR(obj)); \
+    if (JS_VALUE_HAS_REF_COUNT(obj)) { \
+        JSRefCountHeader *p = (JSRefCountHeader *)JS_VALUE_GET_PTR(obj); \
+        fprintf(stderr, "ref_count=%d\n", p->ref_count); \
+    }
+
 #else
 
 #define RETURN_JS(obj) return obj
+#define REF_JS(obj)
 
 #endif
 
@@ -37,12 +56,10 @@ int quickjs_eval_boolback(struct ecmascript_interpreter *interpreter, struct str
 
 void quickjs_call_function(struct ecmascript_interpreter *interpreter, JSValueConst fun, struct string *ret);
 
-#ifndef JS_NAN_BOXING
-inline int operator<(JSValueConst a, JSValueConst b)
-{
-	return JS_VALUE_GET_PTR(a) < JS_VALUE_GET_PTR(b);
+extern struct module quickjs_module;
+
+#ifdef __cplusplus
 }
 #endif
 
-extern struct module quickjs_module;
 #endif
