@@ -3,8 +3,16 @@
 
 #include <quickjs/quickjs.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct form_state;
 
 JSValue js_get_input_object(JSContext *ctx, struct form_state *fs);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
