@@ -1,9 +1,16 @@
-
 #ifndef EL__ECMASCRIPT_QUICKJS_SCREEN_H
 #define EL__ECMASCRIPT_QUICKJS_SCREEN_H
 
 #include <quickjs/quickjs.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int js_screen_init(JSContext *ctx);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
