@@ -11,6 +11,6 @@ JSObject *getElement(JSContext *ctx, void *node);
 
 void walk_tree(struct string *buf, void *nod, bool start = true, bool toSortAttrs = false);
 
-void check_element_event(void *elem, const char *event_name, struct term_event *ev);
+void check_element_event(void *interpreter, void *elem, const char *event_name, struct term_event *ev);
 
 #endif
