@@ -10,6 +10,6 @@ extern JSClass forms_class;
 extern const spidermonkeyFunctionSpec forms_funcs[];
 extern JSPropertySpec forms_props[];
 
-JSObject *get_form_object(JSContext *ctx, JSObject *jsdoc, struct form *form);
+JSObject *get_form_object(JSContext *ctx, struct form *form);
 
 #endif
