@@ -6,13 +6,14 @@
 #include "ecmascript/spidermonkey.h"
 
 struct heartbeat {
-        LIST_HEAD(struct heartbeat);
+        LIST_HEAD_EL(struct heartbeat);
 
         int ttl; /* Time to live.  This value is assigned when the
                   * script begins execution and is decremented every
                   * second.  When it reaches 0, script execution is
                   * terminated. */
 
+        int ref_count;
         struct ecmascript_interpreter *interpreter;
 };
 
