@@ -10,9 +10,11 @@
 #define USE_LIBEVENT
 #endif
 
-#if (defined(HAVE_EVENT_H) || defined(HAVE_EV_EVENT_H) || defined(HAVE_LIBEV_EVENT_H)) && defined(HAVE_LIBEVENT) && !defined(OPENVMS) && !defined(DOS)
-#if defined(HAVE_EVENT_H)
-#include <event.h>
+#if (defined(HAVE_EVENT2_EVENT_H) || defined(HAVE_EV_EVENT_H) || defined(HAVE_LIBEV_EVENT_H)) && defined(HAVE_LIBEVENT) && !defined(OPENVMS) && !defined(DOS)
+#if defined(HAVE_EVENT2_EVENT_H)
+#include <event2/event.h>
+#include <event2/event_compat.h>
+#include <event2/event_struct.h>
 #elif defined(HAVE_EV_EVENT_H)
 #include <ev-event.h>
 #endif
